<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use DateTime;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class Pla01900ParamCozCentral extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $entityManager;
    protected $util;
    protected $Coz01000ConsumoPlanejado;
    protected $novoCodigo;
    protected $environment;
    
    public function __construct($crudService, $parameterBag, $entityManager, $util, $Coz01000ConsumoPlanejado, $novoCodigo, $environment){
        parent::__construct($crudService, $parameterBag);
        $this->entityManager            = $entityManager;
        $this->util                     = $util;
        $this->Coz01000ConsumoPlanejado = $Coz01000ConsumoPlanejado;
        $this->novoCodigo               = $novoCodigo;
        $this->environment              = $environment;
    }
    
    public function validaCadastro(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $row['NRORG'];

        if ($row['CENTRALCELULAPROD'] == 'S') { //validações central
            $retornoCentral = $this->validaCentral($request, $response);
            if(count($retornoCentral) > 0) {
                foreach($retornoCentral as $pos) {
                    if ($pos['CDSERVRETPME'] == NULL && $pos['CDTPRETIPADR'] == NULL) $msg[] = array('msg' => 'CENT_RETAUT_RETDEFAULT', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                    else {
                        if ($pos['CDSERVRETPME'] == NULL) $msg[] = array('msg' => 'CENT_SEM_SERV_RETAUT', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                        if ($pos['CDSERVRETPME'] == NULL) $msg[] = array('msg' => 'CENT_SEM_RETIRADA_DEFAULT', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                    }
                    
                    if ($pos['CDSERVRETPME'] != NULL && $pos['IDSERVATIV'] == 'N')  $msg[] = array('msg' => 'IDSERVATIV_N', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                    if ($pos['CDTPDISTRSAID'] == NULL) $msg[] = array('msg' => 'CENT_SEM_TIPOOP', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                }

                $nrLista = $this->geraListaUnidade($row, $nrorg); //gera lista das unidades 

                $retornoListaUnid = $this->validaUnidAssociadaCentral($nrLista);
                if (count($retornoListaUnid) > 0) {
                    foreach($retornoListaUnid as $pos) {
                        $msg[] = array('msg' => 'UNIDADE_SEM_CENTRAL_CELULAPROD', 'param' => $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL']);
                    }
                }
                    
            } else $msg[] = array('msg' => 'SEM_CENTRAL', 'param' => '');
        }
        
        //Monta lista de servico
        if ($row['CDSERVFILI'] == 'T'){
            $servicos = $this->todosServicos($row, $nrorg);
            $nrListaServ =  $this->Coz01000ConsumoPlanejado->gerarListaServ($servicos, $nrorg);
        } else $nrListaServ =  $this->Coz01000ConsumoPlanejado->gerarListaServ($row['CDSERVFILI'], $nrorg);
        
        if ($row['SERVICOVALID'] == 'S') { //validacoes servico
            $retornoListaServ = $this->validaServico($nrListaServ);

            foreach($retornoListaServ as $pos) {
                if ($pos['IDSERVTRAN'] == 'N') $msg[] = array('msg' => 'SERV_N_TRANSP', 'param' => $pos['CDSERVFILI'] . ' - ' . $pos['NMSERVICO'] . ' (' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ')');
                if ($pos['IDSERVATIV'] == 'N') $msg[] = array('msg' => 'SERV_N_HABILITADO', 'param' => $pos['CDSERVFILI'] . ' - ' . $pos['NMSERVICO'] . ' (' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ')');
            }
        }
        
        if ($row['PRODACABADO'] == 'S') { //validacoes produto acabado
            $resultProdAcabPerCap = $this->validaProdAcabPerCap($nrListaServ, $row);
            foreach ($resultProdAcabPerCap as $pos) {
                $tipoRece = '';
                if ($pos['CDPRATO'] && $pos['CDPRODACABADO'] == NULL) {
                    if ($pos['IDTIPORECE'] == 'N') $msg[] = array('msg' => 'PRATO_SEM_REL_NENHUM', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO']);
                    if ($pos['IDTIPORECE'] == 'P') $msg[] = array('msg' => 'PRATO_SEM_REL_PADRAO', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO']);
                    if ($pos['IDTIPORECE'] == 'F' || $pos['IDTIPORECE'] == 'L') $msg[] = array('msg' => 'PRATO_SEM_REL_LOCAL', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO']. ' (' . $pos['CDFILIAL'] . ' - ' . $pos['NMFILIAL'] . ')');
                    if ($pos['IDTIPORECE'] == 'T') $msg[] = array('msg' => 'PRATO_SEM_REL_TPCARD', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO']);
                    if ($pos['IDTIPORECE'] == 'S') $msg[] = array('msg' => 'PRATO_SEM_REL_SERV', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO'] . ' (' . $pos['CDFILIAL'] . ' - '. $pos['NMFILIAL']  .')'); 
                }
            }
        }
        
        if ($row["QTDPERCAP"] == 'S') { //validacoes qtd per capita
            $resultProdAcabPerCap = $this->validaProdAcabPerCap($nrListaServ, $row);
            foreach ($resultProdAcabPerCap as $pos) {
                if ($pos['QTPORPESSOA'] == NULL || $pos['QTPORPESSOA'] == '0') $msg[] = array('msg' => 'PRATO_SEM_QTD_PES', 'param' => $pos['CDPRATO'] . ' - ' . $pos['NMPRATO']);
            }
        }
        
        if (isset($msg) == true) $response->addDataset(new DataSet('msg', $msg));
        else  {
            $sucesso[] = 'Validação realizada com sucesso.';
            $response->addDataset(new DataSet('sucesso', array_unique($sucesso)));
        }
    }
    
    public function validaCentral($request, $response) {
        $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

        $queryOracle = <<<SQL
            SELECT 
                F.CDFILIAL, 
                F.NMFILIAL, 
                P.CDTPRETIPADR, 
                P.CDSERVRETPME, 
                S.NMSERVICO, 
                S.IDSERVATIV, 
                P.CDTPDISTRSAID 
            FROM 
                FILIAL F 
            INNER JOIN 
                PARAMPLANEJA P ON P.CDFILIAL = F.CDFILIAL 
            LEFT JOIN 
                SERVICO S ON S.CDFILIAL = P.CDFILIAL AND S.CDSERVFILI = P.CDSERVRETPME 
            WHERE 
                F.IDTPFILIALCENT = 'C'
SQL;
    
        $queryPostgre = <<<SQL
            SELECT 
                F.CDFILIAL AS "CDFILIAL",
                F.NMFILIAL AS "NMFILIAL",
                P.CDTPRETIPADR AS "CDTPRETIPADR",
                P.CDSERVRETPME AS "CDSERVRETPME",
                S.NMSERVICO AS "NMSERVICO",
                S.IDSERVATIV AS "IDSERVATIV",
                P.CDTPDISTRSAID AS "CDTPDISTRSAID"
            FROM 
                FILIAL F 
            INNER JOIN 
                PARAMPLANEJA P ON P.CDFILIAL = F.CDFILIAL 
            LEFT JOIN 
                SERVICO S ON S.CDFILIAL = P.CDFILIAL AND S.CDSERVFILI = P.CDSERVRETPME 
            WHERE 
                F.IDTPFILIALCENT = 'C'
SQL;

        $params = array();
    
        $arrayQueries = array(
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
    
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
    }
	
	public function validaUnidAssociadaCentral($nrLista) {
	    $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
	    
	    $queryOracle = <<<SQL
                        SELECT 
                            F.CDFILIAL, 
                            F.NMFILIAL 
                        FROM 
                            FILIAL F 
                        INNER JOIN 
                            LISTA L ON L.NRLISTA = :P_NRLISTA AND L.CDLISTA = F.CDFILIAL 
                        LEFT JOIN 
                            FILIAL C ON C.CDFILIAL = F.CDFILCENTPME AND C.IDTPFILIALCENT = 'C' 
                        WHERE 
                            F.CDFILCENTPME IS NULL
                        AND 
                            F.CDFILIAL NOT IN (
                                SELECT DISTINCT 
                                    CDFILIAGREGA 
                                FROM 
                                    LISTA, 
                                    CELULAPROD 
                                WHERE 
                                    NRLISTA = :P_NRLISTA 
                                AND 
                                    CDFILIAGREGA = CDLISTA
                            )
                        UNION 
                        SELECT 
                            L.CDLISTA, 
                            F.NMFILIAL 
                        FROM 
                            LISTA L 
                        INNER JOIN 
                            FILIAL F ON F.CDFILIAL = L.CDLISTA 
                        LEFT JOIN 
                            CELULAPROD C ON C.CDFILIAGREGA = L.CDLISTA 
                        WHERE 
                            L.NRLISTA = :P_NRLISTA 
                        AND 
                            C.CDFILIAGREGA IS NULL 
                        AND 
                            L.CDLISTA NOT IN (
                                SELECT 
                                    CDFILIAL 
                                FROM 
                                    FILIAL 
                                WHERE 
                                    CDFILCENTPME IS NOT NULL
                            )
SQL;

        $queryPostgre = <<<SQL
            SELECT 
                F.CDFILIAL AS "CDFILIAL", 
                F.NMFILIAL AS "NMFILIAL" 
            FROM 
                FILIAL F 
            INNER JOIN 
                LISTA L ON L.NRLISTA = :P_NRLISTA AND L.CDLISTA = F.CDFILIAL 
            LEFT JOIN 
                FILIAL C ON C.CDFILIAL = F.CDFILCENTPME AND C.IDTPFILIALCENT = 'C' 
            WHERE 
                F.CDFILCENTPME IS NULL
            AND 
                F.CDFILIAL NOT IN (
                    SELECT DISTINCT 
                        CDFILIAGREGA 
                    FROM 
                        LISTA, 
                        CELULAPROD 
                    WHERE 
                        NRLISTA = :P_NRLISTA 
                    AND 
                        CDFILIAGREGA = CDLISTA
                )
            UNION 
            SELECT 
                L.CDLISTA, 
                F.NMFILIAL 
            FROM 
                LISTA L 
            INNER JOIN 
                FILIAL F ON F.CDFILIAL = L.CDLISTA 
            LEFT JOIN 
                CELULAPROD C ON C.CDFILIAGREGA = L.CDLISTA 
            WHERE 
                L.NRLISTA = :P_NRLISTA 
            AND 
                C.CDFILIAGREGA IS NULL 
            AND 
                L.CDLISTA NOT IN (
                    SELECT 
                        CDFILIAL 
                    FROM 
                        FILIAL 
                    WHERE 
                        CDFILCENTPME IS NOT NULL
                )
SQL;

	    $params = array (
            "P_NRLISTA" => $nrLista
        );
        
        $arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
        
        // $validUnid = $this->entityManager->getConnection()->fetchAll($sql, $params);   
        // return $validUnid;
	}
	
	public function validaServico($nrListaServ) {
	    $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
	    
	    $queryOracle = <<<SQL
                        SELECT 
                            L.CDFILIAL, 
                            F.NMFILIAL, 
                            L.CDSERVFILI, 
                            L.NMSERVICO, 
                            S.IDSERVATIV, 
                            S.IDSERVTRAN 
                        FROM 
                            LISTA_SERVICO L 
                        INNER JOIN 
                            SERVICO S ON S.CDFILIAL = L.CDFILIAL AND S.CDSERVFILI = L.CDSERVFILI 
                        INNER JOIN 
                            FILIAL F ON F.CDFILIAL = L.CDFILIAL 
                        WHERE 
                            L.NRLISTA = :P_NRLISTASERV
SQL;

	    
	    $queryPostgre = <<<SQL
                        SELECT 
                            L.CDFILIAL AS "CDFILIAL", 
                            F.NMFILIAL AS "NMFILIAL", 
                            L.CDSERVFILI AS "CDSERVFILI", 
                            L.NMSERVICO AS "NMSERVICO", 
                            S.IDSERVATIV AS "IDSERVATIV", 
                            S.IDSERVTRAN AS "IDSERVTRAN" 
                        FROM 
                            LISTA_SERVICO L 
                        INNER JOIN 
                            SERVICO S ON S.CDFILIAL = L.CDFILIAL AND S.CDSERVFILI = L.CDSERVFILI 
                        INNER JOIN 
                            FILIAL F ON F.CDFILIAL = L.CDFILIAL 
                        WHERE 
                            L.NRLISTA = :P_NRLISTASERV
SQL;

	    $params = array (
            "P_NRLISTASERV" => $nrListaServ
        );
        
        $arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
        
        // $validServ = $this->entityManager->getConnection()->fetchAll($sql, $params);   
        // return $validServ;
	}
	
	public function validaProdAcabPerCap($nrListaServ, $row) {
	    //Utiliza a função do banco de dados 'RECEITA_UTIL' e essa função do banco precisa ser convertida ou adicionada no banco.
	    // Os comentários abaixo já estão na estrutura correta para quando for atualizada a RECEITA_UTIL.
	    
	   // $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
	    
	    $queryOracle = <<<SQL
                        SELECT
                            C.CDFILIAL,
                            C.CDSERVFILI,
                            C.CDTIPOPRAT,
                            R.CDPRATO,
                            C.NMPRATO,
                            R.CDFILAUXPRAT,
                            R.IDTIPORECE,
                            R.CDPRODPRODUCAO AS CDPRODACABADO,
                            MAX(ROUND(R.QTPADRAO, 5)) AS QTPORPESSOA,
                            C.NMFILIAL,
                            C.NMSERVICO
                        FROM (
                            SELECT
                                C.CDFILIAL,
                                C.CDSERVFILI,
                                C.DTCARDAPIO,
                                P.CDTIPOPRAT,
                                P.CDFILAUXPRAT,
                                P.CDPRATO,
                                PT.NMPRATO,
                                F.NMFILIAL,
                                S.NMSERVICO
                            FROM
                                CARDADIA C
                            INNER JOIN
                                LISTA_SERVICO L ON L.NRLISTA = :P_NRLISTASERV AND C.CDFILIAL = L.CDFILIAL AND C.CDSERVFILI = L.CDSERVFILI
                            INNER JOIN
                                COMPCARD P ON C.CDFILIAL = P.CDFILIAL AND P.NRCARDAPIO = C.NRCARDAPIO AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD
                            INNER JOIN
                                PRATO PT ON PT.CDPRATO = P.CDPRATO AND PT.CDFILAUXPRAT = P.CDFILAUXPRAT
                            INNER JOIN
                                FILIAL F ON F.CDFILIAL = C.CDFILIAL
                            INNER JOIN
                                SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI
                            WHERE
                                NVL(C.QTCLIESTICAR, 0) > 0 AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD
                        ) C
                        LEFT JOIN
                            TABLE (RECEITA_UTIL(C.CDFILIAL, C.CDSERVFILI, C.CDPRATO, C.CDFILAUXPRAT, '0', 'Z', NULL, 0, 0, 'S')) R ON R.CDPRATO = C.CDPRATO AND R.CDFILAUXPRAT = C.CDFILAUXPRAT
                        GROUP BY
                            C.CDFILIAL,
                            C.CDSERVFILI,
                            C.CDTIPOPRAT,
                            C.NMPRATO,
                            R.CDPRATO,
                            R.CDFILAUXPRAT,
                            R.IDTIPORECE,
                            R.CDPRODPRODUCAO,
                            C.NMFILIAL,
                            C.NMSERVICO
                        ORDER BY
                            C.CDFILIAL,
                            C.CDSERVFILI,
                            C.CDTIPOPRAT,
                            R.CDPRATO,
                            R.CDFILAUXPRAT
SQL;

	    
// 	    $queryPostgre = <<<SQL
//                         SELECT
//                             C.CDFILIAL AS "CDFILIAL",
//                             C.CDSERVFILI AS "CDSERVFILI",
//                             C.CDTIPOPRAT AS "CDTIPOPRAT",
//                             R.CDPRATO AS "CDPRATO",
//                             C.NMPRATO AS "NMPRATO",
//                             R.CDFILAUXPRAT AS "CDFILAUXPRAT",
//                             R.IDTIPORECE AS "IDTIPORECE",
//                             R.CDPRODPRODUCAO AS "CDPRODACABADO",
//                             ROUND(MAX(R.QTPADRAO)::numeric, 5) AS "QTPORPESSOA",
//                             C.NMFILIAL AS "NMFILIAL",
//                             C.NMSERVICO AS "NMSERVICO"
//                         FROM (
//                             SELECT
//                                 C.CDFILIAL,
//                                 C.CDSERVFILI,
//                                 C.DTCARDAPIO,
//                                 P.CDTIPOPRAT,
//                                 P.CDFILAUXPRAT,
//                                 P.CDPRATO,
//                                 PT.NMPRATO,
//                                 F.NMFILIAL,
//                                 S.NMSERVICO
//                             FROM
//                                 CARDADIA C
//                             INNER JOIN
//                                 LISTA_SERVICO L ON L.NRLISTA = :P_NRLISTASERV AND C.CDFILIAL = L.CDFILIAL AND C.CDSERVFILI = L.CDSERVFILI
//                             INNER JOIN
//                                 COMPCARD P ON C.CDFILIAL = P.CDFILIAL AND P.NRCARDAPIO = C.NRCARDAPIO AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD
//                             INNER JOIN
//                                 PRATO PT ON PT.CDPRATO = P.CDPRATO AND PT.CDFILAUXPRAT = P.CDFILAUXPRAT
//                             INNER JOIN
//                                 FILIAL F ON F.CDFILIAL = C.CDFILIAL
//                             INNER JOIN
//                                 SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI
//                             WHERE
//                                 COALESCE(C.QTCLIESTICAR, 0) > 0 AND C.DTCARDAPIO BETWEEN :P_DTINICARD AND :P_DTFINCARD
//                         ) C
//                         LEFT JOIN LATERAL (
//                             SELECT
//                                 CDPRATO,
//                                 CDFILAUXPRAT,
//                                 IDTIPORECE,
//                                 CDPRODPRODUCAO,
//                                 QTPADRAO
//                             FROM
//                                 RECEITA_UTIL(C.CDFILIAL, C.CDSERVFILI, C.CDPRATO, C.CDFILAUXPRAT, '0', 'Z', NULL, 0, 0, 'S')
//                         ) R ON R.CDPRATO = C.CDPRATO AND R.CDFILAUXPRAT = C.CDFILAUXPRAT
//                         GROUP BY
//                             C.CDFILIAL,
//                             C.CDSERVFILI,
//                             C.CDTIPOPRAT,
//                             C.NMPRATO,
//                             R.CDPRATO,
//                             R.CDFILAUXPRAT,
//                             R.IDTIPORECE,
//                             R.CDPRODPRODUCAO,
//                             C.NMFILIAL,
//                             C.NMSERVICO
//                         ORDER BY
//                             C.CDFILIAL,
//                             C.CDSERVFILI,
//                             C.CDTIPOPRAT,
//                             R.CDPRATO,
//                             R.CDFILAUXPRAT
// SQL;

	    $params = array (
            "P_NRLISTASERV" => $nrListaServ,
            "P_DTINICARD"   => $row['DATAINICIAL'],
            "P_DTFINCARD"   => $row['DATAFINAL']
        );
        
        // $arrayQueries = array(
        //     'default'   => $queryOracle,
        //     'pdo_pgsql' => $queryPostgre
        // );
    
        // $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        // return $connection->fetchAll($query, $params);
        
        $prodAcabPerCap = $this->entityManager->getConnection()->fetchAll($queryOracle, $params);  
        return $prodAcabPerCap;
	}
	
	public function todasUnidades($row, $nrorg) {
	    $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
	    
	    $queryOracle = <<<SQL
                        SELECT DISTINCT 
                            F.CDFILIAL, 
                            F.NMFILIAL, 
                            F.NRORG 
                        FROM 
                            FILIAL F 
                        LEFT JOIN 
                            FILGRUPFILI FG ON F.CDFILIAL = FG.CDFILIAL 
                        WHERE 
                            (
                                (FG.CDGRUPFILI = :CDGRUPFILI)
                                OR 
                                (:CDGRUPFILI = 'T')
                            ) 
                        AND 
                            F.IDCNTRRESC = 'N' 
                        AND 
                            F.NRORG = :NRORG 
                        ORDER BY 
                            F.CDFILIAL
SQL;

	    
	    $queryPostgre = <<<SQL
                        SELECT DISTINCT 
                            F.CDFILIAL AS "CDFILIAL", 
                            F.NMFILIAL AS "NMFILIAL", 
                            F.NRORG AS "NRORG" 
                        FROM 
                            FILIAL F 
                        LEFT JOIN 
                            FILGRUPFILI FG ON F.CDFILIAL = FG.CDFILIAL 
                        WHERE 
                            (
                                (FG.CDGRUPFILI = :CDGRUPFILI) 
                                OR 
                                (:CDGRUPFILI = 'T')
                            ) 
                        AND 
                            F.IDCNTRRESC = 'N' 
                        AND 
                            F.NRORG = :NRORG 
                        ORDER BY 
                            F.CDFILIAL
SQL;

	    
	    $params = array (
            "NRORG"      => $nrorg,
            "CDGRUPFILI" => $row['CDGRUPFILI']
        );
        $arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
        
	   // $result = $this->entityManager->getConnection()->fetchAll($sql, $params);  
    //     return $result;
	}
	
	public function todosServicos($row, $nrorg) { 	   
	    $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();

	    $queryOracle = <<<SQL
                        SELECT DISTINCT 
                            (S.CDFILIAL || S.CDSERVFILI) AS CDFILISERV, 
                            S.CDFILIAL, 
                            F.NMFILIAL, 
                            S.CDSERVFILI, 
                            S.NMSERVICO, 
                            S.NRORG 
                        FROM 
                            SERVICO S 
                        JOIN 
                            FILIAL F ON S.CDFILIAL = F.CDFILIAL 
                        LEFT JOIN 
                            FILGRUPFILI FG ON S.CDFILIAL = FG.CDFILIAL AND (FG.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T') 
                        WHERE 
                            ((INSTR(:CDFILIAL, S.CDFILIAL) > 0) OR (:CDFILIAL = 'T')) 
                            AND ((INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) 
                            AND (IDTIPOSERV = 'N') 
                        ORDER BY 
                            CDFILIAL, 
                            CDSERVFILI
SQL;

	    
	    $queryPostgre = <<<SQL
                        SELECT DISTINCT 
                            (S.CDFILIAL || S.CDSERVFILI) AS "CDFILISERV", 
                            S.CDFILIAL AS "CDFILIAL", 
                            F.NMFILIAL AS "NMFILIAL", 
                            S.CDSERVFILI AS "CDSERVFILI", 
                            S.NMSERVICO AS "NMSERVICO", 
                            S.NRORG AS "NRORG" 
                        FROM 
                            SERVICO S 
                        JOIN 
                            FILIAL F ON S.CDFILIAL = F.CDFILIAL 
                        LEFT JOIN 
                            FILGRUPFILI FG ON S.CDFILIAL = FG.CDFILIAL AND (FG.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T') 
                        WHERE 
                            (POSITION(S.CDFILIAL IN :CDFILIAL) > 0 OR :CDFILIAL = 'T') 
                            AND (POSITION(S.CDGRUPSERV IN CDGRUPSERV) <> 0 OR :CDGRUPSERV = 'T') 
                            AND (S.IDTIPOSERV = 'N') 
                        ORDER BY 
                            S.CDFILIAL, 
                            S.CDSERVFILI
SQL;

	    if (is_array($row['CDFILIAL'])) $unidade = implode("_", $row['CDFILIAL']);
	    else $unidade = $row['CDFILIAL'];
	    
	    $params = array (
	        "CDGRUPSERV" => $row['CDGRUPSERV'],
            "CDFILIAL"   => $unidade,
            "CDGRUPFILI" => $row['CDGRUPFILI']
        );
        
        $arrayQueries = array (
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        return $connection->fetchAll($query, $params);
	}
	
	public function geraListaUnidade($row, $nrorg) {
	    $connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
	    
	    $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab());
        $queryOracle = <<<SQL
                        INSERT INTO LISTA (NRLISTA, CDLISTA)
                        VALUES (:NRLISTA, :CDLISTA)
SQL;

        $queryPostgre = <<<SQL
                        INSERT INTO LISTA (NRLISTA, CDLISTA)
                        VALUES (:NRLISTA, :CDLISTA)
SQL;

        
        $arrayQueries = array(
            'default'   => $queryOracle,
            'pdo_pgsql' => $queryPostgre
        );
        
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        
        if(empty($row['NMFILIAL']) == true || $row['NMFILIAL'] == 'T') {
            $unidade = $this->todasUnidades($row, $nrorg);
            foreach ($unidade as $pos) {
    			$stmt = $this->entityManager->getConnection()->prepare($query);
    			$stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $pos['CDFILIAL']
                    )
                );
    		}
        } else {
    		foreach ($row['NMFILIAL'] as $pos) {
    			$stmt = $this->entityManager->getConnection()->prepare($query);
    			$stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $pos
                    )
                );
    		}
        }
        return $nrlista;
	}
}